/*
 * Copyright (C) 2019 iopsys Software Solutions AB. All rights reserved.
 *
 * Author: <Kenneth.Johansson@iopsys.eu>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA
 */
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>

#include "cmd.h"


struct cmd *fork_cmd(const char *name, const char* const argv[])
{
        pid_t cpid;
        int pipefd[2];
        int c_in, c_out;

        struct cmd *cmd;

        cmd = malloc(sizeof(struct cmd));

        if (NULL == cmd){
                fprintf(stderr, "Memory alloc fail. Now way to continue\n");
                exit(EXIT_FAILURE);
        }

        if (pipe(pipefd) == -1) {
                perror("pipe");
                exit(EXIT_FAILURE);
        }
        cmd->io_in  = pipefd[0];   /* parent. read data from child */
        c_out       = pipefd[1];   /* child. write data to parent  */

        if (pipe(pipefd) == -1) {
                perror("pipe");
                exit(EXIT_FAILURE);
        }
        c_in        = pipefd[0];   /* child. read data from parent */
        cmd->io_out = pipefd[1];   /* parent. write data to child  */

        cpid = fork();
        if (cpid == -1) {
                perror("fork");
                exit(EXIT_FAILURE);
        }

        if (cpid == 0) {
                /* child */
                int fd;
                int maxfd=sysconf(_SC_OPEN_MAX);

                dup2(c_in, STDIN_FILENO);       // redirect stdin to pipe from parent
                dup2(c_out, STDOUT_FILENO);     // redirect stdout to pipe from parent

                for(fd=3; fd<maxfd; fd++)       // close everyhting but 0,1,2
                        close(fd);

                if ( -1 == execvp(name, (char * const*)argv)){
                        fprintf(stderr, "Error starting command [%s]\n",name);
                        perror(NULL);
                        exit(EXIT_FAILURE);
                }

        } else { /* parent */
                close(c_in);
                close(c_out);
                cmd->cpid = cpid;
                return cmd;
        }

        /*can not actually get here but make the compiler happy anyway */
        return NULL;
}
