/*
 * Copyright (C) 2019 iopsys Software Solutions AB. All rights reserved.
 *
 * Author: <Kenneth.Johansson@iopsys.eu>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA
 */
#ifndef _CONSUMER_H_
#define _CONSUMER_H_
#include "list.h"
#include "image.h"

typedef int (*init_t)(struct img_header *hdr);
typedef int (*data_t)(char *buf, int offset, int len);
typedef void (*usage_t)(void);
typedef void (*opt_t)(int argc, char **argv);
typedef void (*finish_t)(void);
typedef void (*post_t)(void);

struct ops {
        struct list_head        list;
        const char              *description;
        void                    *priv;
        init_t                  init;
        data_t                  data;
        usage_t                 usage;
        opt_t                   opt;
        finish_t                finish;
        post_t                  post;
};

/* all consusmers on this list */
extern struct list_head consumers;

void register_consumer(struct ops *ops);

/* WARN!!!     gcc extention.
CONSUMER takes a pointer to a struct ops
the function register_consumer will be called with this
struct filled in before main() is started.
*/
#define CONSUMER(x)                                     \
        static void __attribute__((constructor))        \
        register_##x(void) {                            \
                register_consumer(&x);                  \
        }

#endif /*_CONSUMER_H_*/
