/*
 * Copyright (C) 2019 iopsys Software Solutions AB. All rights reserved.
 *
 * Author: <Kenneth.Johansson@iopsys.eu>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA
 */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <getopt.h>

#include "image.h"
#include "consumer.h"
#include "cmd.h"

static int force;

#define MAX_KEYS 20
static char * key_name[MAX_KEYS];
static char * key_value[MAX_KEYS];
static int keys;

static int init(struct img_header *hdr)
{
        char *p;
        int index;

        /* skip snity check ? */
        if (force) return 0;

        /* Loop over all headers and compare values */
        for(index=0; index<keys; index++) {
                p = get_str_val(hdr, key_name[index]);
                if (p == NULL){
                        fprintf(stderr, "Could not find header [%s] in y3 image\n",key_name[index]);
                        fprintf(stderr, "To skip test give -q (unsafe)\n");
                        exit(EXIT_FAILURE);
                }
                if (strcmp(p,key_value[index])){
                        fprintf(stderr, "Wrong value of header [%s] in y3 image, got [%s] wanted [%s]\n",
                                key_name[index],p,key_value[index]);
                        fprintf(stderr, "To skip test give -q (unsafe)\n");
                        exit(EXIT_FAILURE);
                }
        }
        return 0;
}

static void usage(void)
{
        printf("      -M, --match=key=val,.... image headers needs to have this value\n");
        printf("      -q, --force\tdo not do header check\n");
}


static void parse_match(char *s)
{
        int tot_len= strlen(s);
        int index=0;
        char *p=s,tmp;

        /* find number of keys =  1 + number of , chars in string */
        keys = 1;
        for (int i=0; i<tot_len; i++){
                if (',' == s[i] )
                        keys++;
        }

        if (keys > MAX_KEYS ) {
                fprintf(stderr, "To many header values to match, max %d allowed wanted %d\n", MAX_KEYS, keys);
                exit (EXIT_FAILURE);
        }

        while (1){
                char *e;
                char *c;

                /* copy key to index */
                e = strstr(p, "=");
                c = strstr(p, ",");
                if (e == NULL) {
                        if (c) {
                                fprintf(stderr, "Header match error: expected = found , instread\n");
                                goto error_print;
                        }else
                                break;

                        fprintf(stderr, "Header match error: expected = found end of string\n");
                        goto error_print;

                } else if ( c != NULL && e > c) {
                        fprintf(stderr, "Header match error: Found , before = . missing a = ??\n");
                        goto error_print;
                }
                tmp=*e;*e=0;
                key_name[index] = strdup(p);
                *e=tmp;

                /* copy value */
                p=e+1;
                e = strstr(p, "=");
                c = strstr(p, ",");

                /* last value ?*/
                if (c == NULL ) {
                        key_value[index] = strdup(p);
                        index++;
                        break;
                }
                tmp=*c;*c=0;
                key_value[index] = strdup(p);
                *c=tmp;

                p=c+1;
                index++;
        }

        keys=index;

#if 0
        for (int i=0; i<keys;i++){
                printf("%s=%s\n",key_name[i],key_value[i]);
        }
#endif
        return;
error_print:
        fprintf(stderr, "Trying to parse string [%s]\n",s);
        exit (EXIT_FAILURE);
}

static void opt( int argc, char **argv)
{
        int ch;

        while (1) {
                int option_index = 0;
                static struct option long_options[] = {
                        {"match", required_argument, 0, 'M'},
                        {"force", no_argument, 0, 'q'},
                        {0, 0, 0, 0}
                };

                ch = getopt_long(argc, argv, "M:q",
                                 long_options, &option_index);
                if (ch == -1)
                        break;

                switch (ch) {
                case 'M':
                        parse_match(optarg);
                        break;
                case 'q':
                        force = 1;
                        break;
                default:
                        ;
                }
        }
}

static struct ops my_ops = {
        .description = "Match board name with headers",
        .usage = usage,
        .opt = opt,
        .init = init,
};

CONSUMER(my_ops);
